@@ -40,22 +40,24 @@ struct gui {
   int sysexlist_sysex;
   unsigned char sysexlist_autostop;
   
-  unsigned int refresh_mask;
+  unsigned int refresh_mask, callback_refresh_mask;
   struct block *refresh_tracker_block;
   int refresh_tracker_patpos;
   GMutex *refresh_mutex;
 };
 
 enum {
-  GUI_TRANSPOSE_SONG,
-  GUI_TRANSPOSE_BLOCK,
-  GUI_TRANSPOSE_TRACK,
-  GUI_TRANSPOSE_SELECTION
+  GUI_AREA_SONG,
+  GUI_AREA_BLOCK,
+  GUI_AREA_TRACK,
+  GUI_AREA_SELECTION
 };
 
 enum {
   GUI_REFRESH_INSTRUMENT_SHIFT,
   GUI_REFRESH_TEMPO_SHIFT,
+  GUI_REFRESH_WINDOW_TITLE_SHIFT,
+  GUI_REFRESH_STATUSBAR_SHIFT,
   GUI_REFRESH_INFO_SHIFT,
   GUI_REFRESH_TIMER_SHIFT,
   GUI_REFRESH_BLOCKLIST_SHIFT,
@@ -73,6 +75,8 @@ enum {
 enum {
   GUI_REFRESH_INSTRUMENT=(1 << GUI_REFRESH_INSTRUMENT_SHIFT),
   GUI_REFRESH_TEMPO=(1 << GUI_REFRESH_TEMPO_SHIFT),
+  GUI_REFRESH_WINDOW_TITLE=(1 << GUI_REFRESH_WINDOW_TITLE_SHIFT),
+  GUI_REFRESH_STATUSBAR=(1 << GUI_REFRESH_STATUSBAR_SHIFT),
   GUI_REFRESH_INFO=(1 << GUI_REFRESH_INFO_SHIFT),
   GUI_REFRESH_TIMER=(1 << GUI_REFRESH_TIMER_SHIFT),
   GUI_REFRESH_BLOCKLIST=(1 << GUI_REFRESH_BLOCKLIST_SHIFT),
@@ -94,10 +98,13 @@ void gui_close(struct gui *);
 void gui_set_refresh(struct gui *, unsigned int);
 
 /* These can only be called from the GUI thread */
+void gui_set_callback_refresh(struct gui *, unsigned int);
 void gui_refresh(struct gui *);
 void gui_refresh_all(struct gui *);
 void gui_instrument_refresh(struct gui *, gboolean);
 void gui_tempo_refresh(struct gui *);
+void gui_window_title_refresh(struct gui *);
+void gui_statusbar_refresh(struct gui *);
 void gui_info_refresh(struct gui *);
 void gui_timer_refresh(struct gui *, unsigned int);
 void gui_blocklist_refresh(struct gui *, gboolean);
@@ -106,7 +113,7 @@ void gui_playseq_refresh(struct gui *, gboolean);
 void gui_playseq_info_refresh(struct gui *);
 void gui_sectionlist_refresh(struct gui *, gboolean);
 void gui_sectionlist_buttons_refresh(struct gui *);
-void gui_sysexlist_refresh(struct gui *);
+void gui_sysexlist_refresh(struct gui *, gboolean);
 void gui_trackvolumes_refresh(struct gui *);
 /* Refresh the tracker */
 void gui_tracker_refresh(struct gui *);
