@@ -21,7 +21,6 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <pthread.h>
 #include "song.h"
 #include "editor.h"
 #include "player.h"
@@ -30,8 +29,12 @@
 struct editor *editor_open(char *guifilename) {
   struct editor *editor=(struct editor *)calloc(1, sizeof(struct editor));
 
+  /* Set defaults */
   editor->space=1;
 
+  /* Get a pointer to the main thread */
+  editor->thread=g_thread_self();
+
   /* Open a GUI file */
   editor->gui=gui_open(editor, guifilename);
 
@@ -60,6 +63,9 @@ void editor_song_open(struct editor *editor, char *filename) {
   if(filename!=NULL) {
     editor->filename=strdup(filename);
     editor->song=song_load(filename);
+    /* Just allocate a new song if the song couldn't be opened */
+    if(editor->song==NULL)
+      editor->song=song_alloc();
   } else {
     editor->filename=NULL;
     editor->song=song_alloc();
@@ -91,6 +97,7 @@ void editor_close(struct editor *editor) {
   if(editor->song!=NULL)
     song_free(editor->song);
 
+  /* Free the filename */
   if(editor->filename!=NULL)
     free(editor->filename);
 
@@ -108,180 +115,172 @@ void editor_close(struct editor *editor) {
 }
 
 /* Changes the position of the editor and refreshes the GUI accordingly */
-void editor_set_values(struct editor *editor,
-		       int section, int playseq, int position, int block,
-		       int line, int time, int tempochanged) {
+void editor_set_section(struct editor *editor, int section) {
   if(editor==NULL) {
-    fprintf(stderr, "editor_set_values() called with null editor\n");
-    return;
-  }
-
-  if(section!=editor->section &&
-     section>=0 && section<editor->song->numsections) {
-    gui_set_refresh(editor->gui, GUI_REFRESH_INFO|GUI_REFRESH_SECTIONLIST);
-    editor->section=section;
-  }
-  if(playseq!=editor->playseq &&
-     playseq>=0 && playseq<editor->song->numplayseqs) {
-    gui_set_refresh(editor->gui, GUI_REFRESH_INFO|GUI_REFRESH_PLAYSEQ);
-    editor->playseq=playseq;
-  }
-  if(position!=editor->position &&
-     position>=0 && position<editor->song->playseqs[editor->playseq]->length) {
-    gui_set_refresh(editor->gui, GUI_REFRESH_INFO|GUI_REFRESH_PLAYSEQ);
-    editor->position=position;
-  }
-  if(block!=editor->block &&
-     block>=0 && block<editor->song->numblocks) {
-    gui_set_refresh(editor->gui,
-		    GUI_REFRESH_INFO|
-		    GUI_REFRESH_BLOCKLIST|GUI_REFRESH_TRACKER);
-    editor->block=block;
-  }
-  if(line!=editor->line &&
-     line>=0 && line<editor->song->blocks[editor->block]->length) {
-    gui_set_refresh(editor->gui, GUI_REFRESH_TRACKER);
-    editor->line=line;
-  }
-  if(time!=editor->time) {
-    gui_set_refresh(editor->gui, GUI_REFRESH_TIMER);
-    editor->time=time;
-  }
-  if(tempochanged)
-    gui_set_refresh(editor->gui, GUI_REFRESH_TEMPO);
-}
-
-/* Changes the position of the editor and refreshes the GUI accordingly */
-void editor_set_section_immediate(struct editor *editor, int section) {
-  if(editor==NULL) {
-    fprintf(stderr, "editor_set_section_immediate() called with null editor\n");
+    fprintf(stderr, "editor_set_section() called with null editor\n");
     return;
   }
 
   if(section!=editor->section) {
+    /* Bounds checking */
     if(section<0)
       section=0;
-
     if(section>=editor->song->numsections)
       section=editor->song->numsections-1;
-      
+    
     editor->section=section;
-
-    gui_info_refresh(editor->gui); 
-    gui_sectionlist_refresh(editor->gui, FALSE);
+    
+    /* If called from GUI set player position, always refresh GUI */
+    if(g_thread_self()==editor->thread)
+      player_set_section(editor->player, section);
+    gui_set_refresh(editor->gui,
+		    GUI_REFRESH_SECTIONLIST|GUI_REFRESH_INFO|
+		    GUI_REFRESH_WINDOW_TITLE);
   }
 }
 
 /* Changes the position of the editor and refreshes the GUI accordingly */
-void editor_set_playseq_immediate(struct editor *editor, int playseq) {
+void editor_set_playseq(struct editor *editor, int playseq) {
   if(editor==NULL) {
-    fprintf(stderr, "editor_set_playseq_immediate() called with null editor\n");
+    fprintf(stderr, "editor_set_playseq() called with null editor\n");
     return;
   }
 
   if(playseq!=editor->playseq) {
+    /* Bounds checking */
     if(playseq<0)
       playseq=0;
-
     if(playseq>=editor->song->numplayseqs)
       playseq=editor->song->numplayseqs-1;
-
+    
     editor->playseq=playseq;
-
-    gui_info_refresh(editor->gui);
-    gui_playseq_refresh(editor->gui, FALSE);
+    
+    /* If called from GUI set player position, always refresh GUI */
+    if(g_thread_self()==editor->thread)
+      player_set_playseq(editor->player, playseq);
+    gui_set_refresh(editor->gui,
+		    GUI_REFRESH_PLAYSEQ|GUI_REFRESH_INFO|
+		    GUI_REFRESH_WINDOW_TITLE);
   }
 }
 
 /* Changes the position of the editor and refreshes the GUI accordingly */
-void editor_set_position_immediate(struct editor *editor, int position) {
+void editor_set_position(struct editor *editor, int position) {
   if(editor==NULL) {
-    fprintf(stderr, "editor_set_position_immediate() called with null editor\n");
+    fprintf(stderr, "editor_set_position() called with null editor\n");
     return;
   }
 
   if(position!=editor->position) {
+    /* Bounds checking */
     if(position<0)
       position=0;
-
     if(position>=editor->song->playseqs[editor->playseq]->length)
       position=editor->song->playseqs[editor->playseq]->length-1;
-
+    
     editor->position=position;
-
-    gui_info_refresh(editor->gui);
-    gui_playseq_refresh(editor->gui, FALSE);
+    
+    /* If called from GUI set player position, always refresh GUI */
+    if(g_thread_self()==editor->thread)
+      player_set_position(editor->player, position);
+    gui_set_refresh(editor->gui,
+		    GUI_REFRESH_PLAYSEQ|GUI_REFRESH_INFO);
   }
 }
 
 /* Changes the position of the editor and refreshes the GUI accordingly */
-void editor_set_block_immediate(struct editor *editor, int block) {
+void editor_set_block(struct editor *editor, int block) {
   if(editor==NULL) {
-    fprintf(stderr, "editor_set_block_immediate() called with null editor\n");
+    fprintf(stderr, "editor_set_block() called with null editor\n");
     return;
   }
 
   if(block!=editor->block) {
+    /* Bounds checking */
     if(block<0)
       block=0;
-
     if(block>=editor->song->numblocks)
       block=editor->song->numblocks-1;
-
-    if(editor->cmdpage>=editor->song->blocks[block]->commandpages) {
-      while(editor->cmdpage>=editor->song->blocks[block]->commandpages)
-	editor->cmdpage--;
-    }
-
+    while(editor->cmdpage>=editor->song->blocks[block]->commandpages)
+      editor->cmdpage--;
+    
     editor->block=block;
 
-    gui_info_refresh(editor->gui);
-    gui_blocklist_refresh(editor->gui, FALSE);
-    gui_tracker_refresh(editor->gui);
+    /* If called from GUI set player position, always refresh GUI */
+    if(g_thread_self()==editor->thread)
+      player_set_block(editor->player, block);
+    gui_set_refresh(editor->gui,
+		    GUI_REFRESH_BLOCKLIST|GUI_REFRESH_INFO|
+		    GUI_REFRESH_WINDOW_TITLE|GUI_REFRESH_TRACKER);
   }
 }
 
 /* Changes the position of the editor and refreshes the GUI accordingly */
-void editor_set_line_immediate(struct editor *editor, int line) {
+void editor_set_line(struct editor *editor, int line) {
   if(editor==NULL) {
-    fprintf(stderr, "editor_set_line_immediate() called with null editor\n");
+    fprintf(stderr, "editor_set_line() called with null editor\n");
     return;
   }
 
   if(line!=editor->line) {
+    /* Bounds checking */
     while(line<0)
       line+=editor->song->blocks[editor->block]->length;
-
     while(line>=editor->song->blocks[editor->block]->length)
       line-=editor->song->blocks[editor->block]->length;
     
     editor->line=line;
-
-    gui_tracker_refresh(editor->gui);
+    
+    /* If called from GUI set player position, always refresh GUI */
+    if(g_thread_self()==editor->thread) {
+      player_set_line(editor->player, line);
+      gui_tracker_refresh(editor->gui);
+    } else
+      gui_set_refresh(editor->gui, GUI_REFRESH_TRACKER);
   }
 }
 
 /* Changes the position of the editor and refreshes the GUI accordingly */
-void editor_set_commandpage_immediate(struct editor *editor, int cmdpage) {
+void editor_set_commandpage(struct editor *editor, int cmdpage) {
   if(editor==NULL) {
-    fprintf(stderr, "editor_set_commandpage_immediate() called with null editor\n");
+    fprintf(stderr, "editor_set_commandpage() called with null editor\n");
     return;
   }
 
   if(cmdpage!=editor->cmdpage) {
+    /* Bounds checking */
     while(cmdpage<0)
       cmdpage+=editor->song->blocks[editor->block]->commandpages;
-
     while(cmdpage>=editor->song->blocks[editor->block]->commandpages)
       cmdpage-=editor->song->blocks[editor->block]->commandpages;
-
+    
     editor->cmdpage=cmdpage;
     
-    gui_info_refresh(editor->gui);
-    gui_tracker_refresh(editor->gui);
+    gui_set_refresh(editor->gui, GUI_REFRESH_INFO|GUI_REFRESH_TRACKER);
   }
 }
 
+/* Changes the position of the editor and refreshes the GUI accordingly */
+void editor_set_time(struct editor *editor, int time) {
+  if(editor==NULL) {
+    fprintf(stderr, "editor_set_time() called with null editor\n");
+    return;
+  }
+
+  editor->time=time;
+  gui_set_refresh(editor->gui, GUI_REFRESH_TIMER);
+}
+
+/* Changes the position of the editor and refreshes the GUI accordingly */
+void editor_set_tempo(struct editor *editor) {
+  if(editor==NULL) {
+    fprintf(stderr, "editor_set_tempo() called with null editor\n");
+    return;
+  }
+
+  gui_set_refresh(editor->gui, GUI_REFRESH_WINDOW_TITLE);
+}
+
 /* Refreshes playseq from section and block from position */
 void editor_refresh_playseq_and_block(struct editor *editor) {
   struct song *song;
@@ -293,18 +292,29 @@ void editor_refresh_playseq_and_block(struct editor *editor) {
 
   song=editor->song;
 
+  /* Bounds checking */
   if(editor->section>=song->numsections)
     editor->section=0;
 
+  /* Get playseq number */
   editor->playseq=song->sections[editor->section];
 
+  /* Bounds checking */
   if(editor->position>=song->playseqs[editor->playseq]->length) {
     editor->position=0;
     editor_refresh_playseq_and_block(editor);
   } else
     editor->block=song->playseqs[editor->playseq]->blocknumbers[editor->position];
+
+  /* If called from GUI set player position, always refresh GUI */
+  if(g_thread_self()==editor->thread)
+    player_set_playseq(editor->player, editor->playseq);
+  gui_set_refresh(editor->gui,
+		  GUI_REFRESH_PLAYSEQ|GUI_REFRESH_BLOCKLIST|GUI_REFRESH_INFO|
+		  GUI_REFRESH_WINDOW_TITLE);
 }
 
+/* Reads MIDI input */
 void editor_midi_input(struct editor *editor, unsigned char *data) {
   Tracker *tracker;
   struct song *song;
@@ -314,7 +324,7 @@ void editor_midi_input(struct editor *editor, unsigned char *data) {
     return;
   }
 
-  tracker=(Tracker *)glade_xml_get_widget(editor->gui->xml, "gui_tracker");
+  tracker=(Tracker *)glade_xml_get_widget(editor->gui->xml, "tracker");
   song=editor->song;
 
   switch(data[0]&0xf0) {
@@ -325,7 +335,7 @@ void editor_midi_input(struct editor *editor, unsigned char *data) {
 	editor->chordstatus--;
 	tracker_step_cursor_channel(tracker, -1);
 	if(editor->chordstatus==0)
-	  editor_set_line_immediate(editor, editor->line+editor->space);
+	  editor_set_line(editor, editor->line+editor->space);
       }
     }
     break;
@@ -343,12 +353,12 @@ void editor_midi_input(struct editor *editor, unsigned char *data) {
 	editor->chordstatus++;
 	tracker_step_cursor_channel(tracker, 1);
       } else
-	editor_set_line_immediate(editor, editor->line+editor->space);
+	editor_set_line(editor, editor->line+editor->space);
     }
     break;
   case 0xb0:
     /* MIDI controller */
-    if(editor->edit) {
+    if(editor->edit && editor->recordcontrollers) {
       block_set_command_full(song->blocks[editor->block], tracker->patpos,
 			     tracker->cursor_ch, editor->cmdpage,
 			     COMMAND_MIDI_CONTROLLERS+data[1], data[2]);
@@ -368,3 +378,13 @@ void editor_midi_input(struct editor *editor, unsigned char *data) {
     break;
   }
 }
+
+/* Refreshes the GUI when the player has stopped */
+void editor_stop(struct editor *editor) {
+  if(editor==NULL) {
+    fprintf(stderr, "editor_stop() called with null editor\n");
+    return;
+  }
+
+  gui_set_refresh(editor->gui, GUI_REFRESH_STATUSBAR);
+}
