@@ -88,7 +88,7 @@ struct gui *gui_open(struct editor *editor, char *filename) {
   /* Load the GUI from a GLADE file and connect signals */
   gui->xml=glade_xml_new(filename, NULL, NULL);
   glade_xml_signal_autoconnect_full(gui->xml, gui_gladexmlconnectfunc, gui);
-  gui->window_main=glade_xml_get_widget(gui->xml, "gui_window_main");
+  gui->window_main=glade_xml_get_widget(gui->xml, "window_main");
   gtk_widget_add_events(gui->window_main, GDK_KEY_RELEASE_MASK);
 
   /* Create the MVC classes for various list view widgets */
@@ -98,11 +98,11 @@ struct gui *gui_open(struct editor *editor, char *filename) {
   gui->store_sectionlist=gtk_list_store_new(3, G_TYPE_INT, G_TYPE_INT,
 					    G_TYPE_STRING);
   treeview=GTK_TREE_VIEW(glade_xml_get_widget(gui->xml,
-					      "gui_treeview_sectionlist"));
+					      "treeview_sectionlist"));
   select=gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
   gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);
   g_signal_connect(G_OBJECT (select), "changed",
-		   G_CALLBACK(on_gui_selection_sectionlist_changed),
+		   G_CALLBACK(on_selection_sectionlist_changed),
 		   gui);
   gtk_tree_view_set_model(treeview, GTK_TREE_MODEL(gui->store_sectionlist));
   gtk_tree_view_append_column(treeview,
@@ -118,11 +118,11 @@ struct gui *gui_open(struct editor *editor, char *filename) {
   /* Playing Sequence */
   gui->store_playseq=gtk_list_store_new(3, G_TYPE_INT, G_TYPE_INT,
 					G_TYPE_STRING);
-  treeview=GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "gui_treeview_playseq"));
+  treeview=GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_playseq"));
   select=gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
   gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);
   g_signal_connect(G_OBJECT (select), "changed",
-		   G_CALLBACK(on_gui_selection_playseq_changed),
+		   G_CALLBACK(on_selection_playseq_changed),
 		   gui);
   gtk_tree_view_set_model(treeview, GTK_TREE_MODEL(gui->store_playseq));
   gtk_tree_view_append_column(treeview,
@@ -137,11 +137,11 @@ struct gui *gui_open(struct editor *editor, char *filename) {
 
   /* Block List */
   gui->store_blocklist=gtk_list_store_new(2, G_TYPE_INT, G_TYPE_STRING);
-  treeview=GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "gui_treeview_blocklist"));
+  treeview=GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_blocklist"));
   select=gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
   gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);
   g_signal_connect(G_OBJECT (select), "changed",
-		   G_CALLBACK(on_gui_selection_blocklist_changed),
+		   G_CALLBACK(on_selection_blocklist_changed),
 		   gui);
   gtk_tree_view_set_model(treeview, GTK_TREE_MODEL(gui->store_blocklist));
   gtk_tree_view_append_column(treeview,
@@ -153,11 +153,11 @@ struct gui *gui_open(struct editor *editor, char *filename) {
 
   /* SysEx List */
   gui->store_sysexlist=gtk_list_store_new(2, G_TYPE_INT, G_TYPE_STRING);
-  treeview=GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "gui_treeview_sysexlist"));
+  treeview=GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_sysexlist"));
   select=gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
   gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);
   g_signal_connect(G_OBJECT (select), "changed",
-		   G_CALLBACK(on_gui_selection_sysexlist_changed),
+		   G_CALLBACK(on_selection_sysexlist_changed),
 		   gui);
   gtk_tree_view_set_model(treeview, GTK_TREE_MODEL(gui->store_sysexlist));
   gtk_tree_view_append_column(treeview,
@@ -168,14 +168,11 @@ struct gui *gui_open(struct editor *editor, char *filename) {
 					 "text", 1, NULL));
 
   /* Set the keyboard octave */
-  gtk_option_menu_set_history(GTK_OPTION_MENU(glade_xml_get_widget(gui->xml, "gui_optionmenu_keyboard")), 3);
+  gtk_option_menu_set_history(GTK_OPTION_MENU(glade_xml_get_widget(gui->xml, "optionmenu_keyboard")), 3);
   gui->editor->octave=3;
 
   /* Make some widgets that are not usable yet insensitive */
-  gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml,
-						"gui_menuitem_file_print"),
-			   FALSE);
-  gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "gui_menuitem_settings_preferences"), FALSE);
+  gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "menuitem_settings_preferences"), FALSE);
 
   return gui;
 }
@@ -190,7 +187,7 @@ void gui_close(struct gui *gui) {
 /* Refreshes all parts of the GUI */
 void gui_refresh_all(struct gui *gui) {
   /* Reset the tracker */
-  Tracker *tracker=(Tracker *)glade_xml_get_widget(gui->xml, "gui_tracker");
+  Tracker *tracker=(Tracker *)glade_xml_get_widget(gui->xml, "tracker");
 
   tracker_reset(tracker);
   tracker_set_pattern(tracker, gui->editor->song->blocks[gui->editor->block]);
@@ -204,7 +201,7 @@ void gui_refresh_all(struct gui *gui) {
   gui_playseq_refresh(gui, TRUE);
   gui_sectionlist_refresh(gui, TRUE);
   gui_instrument_refresh(gui, TRUE);
-  gui_sysexlist_refresh(gui);
+  gui_sysexlist_refresh(gui, TRUE);
   gui_trackvolumes_refresh(gui);
   gui_song_refresh(gui);
 
@@ -217,62 +214,166 @@ void gui_refresh_all(struct gui *gui) {
 }
  
 Tracker *gui_get_tracker(struct gui *gui) {
-  return (Tracker *)glade_xml_get_widget(gui->xml, "gui_tracker");
+  return (Tracker *)glade_xml_get_widget(gui->xml, "tracker");
 }
 
 int gui_get_tracker_position(struct gui *gui) {
-  Tracker *tracker=(Tracker *)glade_xml_get_widget(gui->xml, "gui_tracker");
+  Tracker *tracker=(Tracker *)glade_xml_get_widget(gui->xml, "tracker");
 
   return tracker->patpos;
 }
 
 /* Set the refresh mask */
 void gui_set_refresh(struct gui *gui, unsigned int mask) {
-  g_mutex_lock(gui->refresh_mutex);
+  if(g_thread_self()!=gui->editor->thread)
+    g_mutex_lock(gui->refresh_mutex);
   gui->refresh_mask|=mask;
-  g_mutex_unlock(gui->refresh_mutex);
+  if(g_thread_self()!=gui->editor->thread)
+    g_mutex_unlock(gui->refresh_mutex);
 }
 
 /* Refresh all those parts of the GUI that need to be refreshed */
 void gui_refresh(struct gui *gui) {
   g_mutex_lock(gui->refresh_mutex);
-  if(gui->refresh_mask&GUI_REFRESH_INSTRUMENT)
+  if(gui->refresh_mask&GUI_REFRESH_INSTRUMENT) {
     gui_instrument_refresh(gui, FALSE);
-  if(gui->refresh_mask&GUI_REFRESH_TEMPO)
-    gui_tempo_refresh(gui);
-  if(gui->refresh_mask&GUI_REFRESH_INFO)
-    gui_info_refresh(gui);
-  if(gui->refresh_mask&GUI_REFRESH_TIMER)
-    gui_timer_refresh(gui, gui->editor->time);
-  if(gui->refresh_mask&GUI_REFRESH_BLOCKLIST)
-    gui_blocklist_refresh(gui, FALSE);
-  if(gui->refresh_mask&GUI_REFRESH_BLOCKLIST_INFO)
-    gui_blocklist_info_refresh(gui);
-  if(gui->refresh_mask&GUI_REFRESH_PLAYSEQ)
-    gui_playseq_refresh(gui, FALSE);
-  if(gui->refresh_mask&GUI_REFRESH_PLAYSEQ_INFO)
-    gui_playseq_info_refresh(gui);
-  if(gui->refresh_mask&GUI_REFRESH_SECTIONLIST)
-    gui_sectionlist_refresh(gui, FALSE);
-  if(gui->refresh_mask&GUI_REFRESH_SECTIONLIST_BUTTONS)
-    gui_sectionlist_buttons_refresh(gui);
-  if(gui->refresh_mask&GUI_REFRESH_SYSEXLIST)
-    gui_sysexlist_refresh(gui);
-  if(gui->refresh_mask&GUI_REFRESH_TRACKVOLUMES)
-    gui_trackvolumes_refresh(gui);
-  if(gui->refresh_mask&GUI_REFRESH_SONG)
-    gui_song_refresh(gui);
-  if(gui->refresh_mask&GUI_REFRESH_TRACKER)
-    gui_tracker_refresh(gui);
-  
-  gui->refresh_mask=0;
+    gui->refresh_mask^=GUI_REFRESH_INSTRUMENT;
+  }
   g_mutex_unlock(gui->refresh_mutex);
+  g_thread_yield();
+  
+  g_mutex_lock(gui->refresh_mutex);
+  if(gui->refresh_mask&GUI_REFRESH_TEMPO) {
+    gui_tempo_refresh(gui);
+    gui->refresh_mask^=GUI_REFRESH_TEMPO;
+  }
+  g_mutex_unlock(gui->refresh_mutex);
+  g_thread_yield();
+  
+  g_mutex_lock(gui->refresh_mutex);
+  if(gui->refresh_mask&GUI_REFRESH_INFO) {
+    gui_info_refresh(gui);
+    gui->refresh_mask^=GUI_REFRESH_INFO;
+  }
+  g_mutex_unlock(gui->refresh_mutex);
+  g_thread_yield();
+  
+  g_mutex_lock(gui->refresh_mutex);
+  if(gui->refresh_mask&GUI_REFRESH_WINDOW_TITLE) {
+    gui_window_title_refresh(gui);
+    gui->refresh_mask^=GUI_REFRESH_WINDOW_TITLE;
+  }
+  g_mutex_unlock(gui->refresh_mutex);
+  g_thread_yield();
+  
+  g_mutex_lock(gui->refresh_mutex);
+  if(gui->refresh_mask&GUI_REFRESH_STATUSBAR) {
+    gui_statusbar_refresh(gui);
+    gui->refresh_mask^=GUI_REFRESH_STATUSBAR;
+  }
+  g_mutex_unlock(gui->refresh_mutex);
+  g_thread_yield();
+  
+  g_mutex_lock(gui->refresh_mutex);
+  if(gui->refresh_mask&GUI_REFRESH_TIMER) {
+    gui_timer_refresh(gui, gui->editor->time);
+    gui->refresh_mask^=GUI_REFRESH_TIMER;
+  }
+  g_mutex_unlock(gui->refresh_mutex);
+  g_thread_yield();
+  
+  g_mutex_lock(gui->refresh_mutex);
+  if(gui->refresh_mask&GUI_REFRESH_BLOCKLIST) {
+    gui_blocklist_refresh(gui, FALSE);
+    gui->refresh_mask^=GUI_REFRESH_BLOCKLIST;
+  }
+  g_mutex_unlock(gui->refresh_mutex);
+  g_thread_yield();
+  
+  g_mutex_lock(gui->refresh_mutex);
+  if(gui->refresh_mask&GUI_REFRESH_BLOCKLIST_INFO) {
+    gui_blocklist_info_refresh(gui);
+    gui->refresh_mask^=GUI_REFRESH_BLOCKLIST_INFO;
+  }
+  g_mutex_unlock(gui->refresh_mutex);
+  g_thread_yield();
+  
+  g_mutex_lock(gui->refresh_mutex);
+  if(gui->refresh_mask&GUI_REFRESH_PLAYSEQ) {
+    gui_playseq_refresh(gui, FALSE);
+    gui->refresh_mask^=GUI_REFRESH_PLAYSEQ;
+  }
+  g_mutex_unlock(gui->refresh_mutex);
+  g_thread_yield();
+  
+  g_mutex_lock(gui->refresh_mutex);
+  if(gui->refresh_mask&GUI_REFRESH_PLAYSEQ_INFO) {
+    gui_playseq_info_refresh(gui);
+    gui->refresh_mask^=GUI_REFRESH_PLAYSEQ_INFO;
+  }
+  g_mutex_unlock(gui->refresh_mutex);
+  g_thread_yield();
+  
+  g_mutex_lock(gui->refresh_mutex);
+  if(gui->refresh_mask&GUI_REFRESH_SECTIONLIST) {
+    gui_sectionlist_refresh(gui, FALSE);
+    gui->refresh_mask^=GUI_REFRESH_SECTIONLIST;
+  }
+  g_mutex_unlock(gui->refresh_mutex);
+  g_thread_yield();
+  
+  g_mutex_lock(gui->refresh_mutex);
+  if(gui->refresh_mask&GUI_REFRESH_SECTIONLIST_BUTTONS) {
+    gui_sectionlist_buttons_refresh(gui);
+    gui->refresh_mask^=GUI_REFRESH_SECTIONLIST_BUTTONS;
+  }
+  g_mutex_unlock(gui->refresh_mutex);
+  g_thread_yield();
+  
+  g_mutex_lock(gui->refresh_mutex);
+  if(gui->refresh_mask&GUI_REFRESH_SYSEXLIST) {
+    gui_sysexlist_refresh(gui, FALSE);
+    gui->refresh_mask^=GUI_REFRESH_SYSEXLIST;
+  }
+  g_mutex_unlock(gui->refresh_mutex);
+  g_thread_yield();
+  
+  g_mutex_lock(gui->refresh_mutex);
+  if(gui->refresh_mask&GUI_REFRESH_TRACKVOLUMES) {
+    gui_trackvolumes_refresh(gui);
+    gui->refresh_mask^=GUI_REFRESH_TRACKVOLUMES;
+  }
+  g_mutex_unlock(gui->refresh_mutex);
+  g_thread_yield();
+  
+  g_mutex_lock(gui->refresh_mutex);
+  if(gui->refresh_mask&GUI_REFRESH_SONG) {
+    gui_song_refresh(gui);
+    gui->refresh_mask^=GUI_REFRESH_SONG;
+  }
+  g_mutex_unlock(gui->refresh_mutex);
+  g_thread_yield();
+  
+  g_mutex_lock(gui->refresh_mutex);
+  if(gui->refresh_mask&GUI_REFRESH_TRACKER) {
+    gui_tracker_refresh(gui);
+    gui->refresh_mask^=GUI_REFRESH_TRACKER;
+  }
+  g_mutex_unlock(gui->refresh_mutex);
+  g_thread_yield();
 }
 
 /* Refresh the tracker */
 void gui_tracker_refresh(struct gui *gui) {
   Tracker *tracker=gui_get_tracker(gui);
 
+  /* Check that the line is valid */
+  if(gui->editor->line>gui->editor->song->blocks[gui->editor->block]->length)
+    gui->editor->line=gui->editor->song->blocks[gui->editor->block]->length-1;
+  /* Check that the commad page is valid */
+  if(gui->editor->cmdpage>gui->editor->song->blocks[gui->editor->block]->commandpages)
+    gui->editor->cmdpage=gui->editor->song->blocks[gui->editor->block]->commandpages-1;
+
   tracker_set_pattern(tracker, gui->editor->song->blocks[gui->editor->block]);
   tracker_set_patpos(tracker, gui->editor->line);
   tracker_set_cmdpage(tracker, gui->editor->cmdpage);
@@ -281,12 +382,14 @@ void gui_tracker_refresh(struct gui *gui) {
 /* Refresh the song properties window widgets */
 void gui_song_refresh(struct gui *gui) {
   if(gui->editor->song->name!=NULL)
-    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(gui->xml, "gui_entry_songname")), gui->editor->song->name);
+    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(gui->xml,
+						      "entry_songname")),
+		       gui->editor->song->name);
 
   if(gui->editor->song->sendsync)
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(gui->xml, "gui_checkbutton_sendsync")), TRUE);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(gui->xml, "checkbutton_sendsync")), TRUE);
   else
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(gui->xml, "gui_checkbutton_sendsync")), FALSE);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(gui->xml, "checkbutton_sendsync")), FALSE);
 }
 
 void gui_instrument_refresh(struct gui *gui, gboolean refresh_spin) {
@@ -296,21 +399,21 @@ void gui_instrument_refresh(struct gui *gui, gboolean refresh_spin) {
   /* Refresh the main window widgets */
   /* Only refresh the spin button if necessary to avoid forever loops */
   if(refresh_spin)
-    gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(gui->xml, "gui_spinbutton_instrument")), gui->editor->instrument+1);
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(gui->xml, "spinbutton_instrument")), gui->editor->instrument+1);
   if(gui->editor->song->instruments[gui->editor->instrument]->name!=NULL)
-    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(gui->xml, "gui_entry_instrument")), gui->editor->song->instruments[gui->editor->instrument]->name);
+    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(gui->xml, "entry_instrument")), gui->editor->song->instruments[gui->editor->instrument]->name);
   else
-    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(gui->xml, "gui_entry_instrument")), "");
+    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(gui->xml, "entry_instrument")), "");
 
   /* Refresh the instrument window widgets */
   if(gui->editor->song->instruments[gui->editor->instrument]->name!=NULL)
-    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(gui->xml, "gui_entry_instrumentproperties_name")), gui->editor->song->instruments[gui->editor->instrument]->name);
+    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(gui->xml, "entry_instrumentproperties_name")), gui->editor->song->instruments[gui->editor->instrument]->name);
   else
-    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(gui->xml, "gui_entry_instrumentproperties_name")), "");
-  gtk_adjustment_set_value(gtk_range_get_adjustment(GTK_RANGE(glade_xml_get_widget(gui->xml, "gui_scale_instrumentproperties_volume"))), gui->editor->song->instruments[gui->editor->instrument]->defaultvelocity);
-  gtk_adjustment_set_value(gtk_range_get_adjustment(GTK_RANGE(glade_xml_get_widget(gui->xml, "gui_scale_instrumentproperties_transpose"))), gui->editor->song->instruments[gui->editor->instrument]->transpose);
-  gtk_adjustment_set_value(gtk_range_get_adjustment(GTK_RANGE(glade_xml_get_widget(gui->xml, "gui_scale_instrumentproperties_hold"))), gui->editor->song->instruments[gui->editor->instrument]->hold);
-  gtk_adjustment_set_value(gtk_range_get_adjustment(GTK_RANGE(glade_xml_get_widget(gui->xml, "gui_scale_instrumentproperties_midich"))), gui->editor->song->instruments[gui->editor->instrument]->midichannel+1);
+    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(gui->xml, "entry_instrumentproperties_name")), "");
+  gtk_adjustment_set_value(gtk_range_get_adjustment(GTK_RANGE(glade_xml_get_widget(gui->xml, "scale_instrumentproperties_volume"))), gui->editor->song->instruments[gui->editor->instrument]->defaultvelocity);
+  gtk_adjustment_set_value(gtk_range_get_adjustment(GTK_RANGE(glade_xml_get_widget(gui->xml, "scale_instrumentproperties_transpose"))), gui->editor->song->instruments[gui->editor->instrument]->transpose);
+  gtk_adjustment_set_value(gtk_range_get_adjustment(GTK_RANGE(glade_xml_get_widget(gui->xml, "scale_instrumentproperties_hold"))), gui->editor->song->instruments[gui->editor->instrument]->hold);
+  gtk_adjustment_set_value(gtk_range_get_adjustment(GTK_RANGE(glade_xml_get_widget(gui->xml, "scale_instrumentproperties_midich"))), gui->editor->song->instruments[gui->editor->instrument]->midichannel+1);
 }
 
 void gui_tempo_refresh(struct gui *gui) {
@@ -318,8 +421,8 @@ void gui_tempo_refresh(struct gui *gui) {
   char *title=(char *)calloc(l, sizeof(char));
 
   /* Refresh the song properties window widgets */
-  gtk_adjustment_set_value(gtk_range_get_adjustment(GTK_RANGE(glade_xml_get_widget(gui->xml, "gui_scale_tempo"))), gui->editor->song->tempo);
-  gtk_adjustment_set_value(gtk_range_get_adjustment(GTK_RANGE(glade_xml_get_widget(gui->xml, "gui_scale_tpl"))), gui->editor->song->ticksperline);
+  gtk_adjustment_set_value(gtk_range_get_adjustment(GTK_RANGE(glade_xml_get_widget(gui->xml, "scale_tempo"))), gui->editor->song->tempo);
+  gtk_adjustment_set_value(gtk_range_get_adjustment(GTK_RANGE(glade_xml_get_widget(gui->xml, "scale_tpl"))), gui->editor->song->ticksperline);
 
   /* Refresh the main window widgets if the window exists */
   if(gui->window_main!=NULL && GTK_IS_WINDOW(gui->window_main)) {
@@ -335,83 +438,84 @@ void gui_tempo_refresh(struct gui *gui) {
   free(title);
 }
 
-void gui_info_refresh(struct gui *gui) {
-  char section[9+8], playseq[10+8], block[7+10+32];
+void gui_window_title_refresh(struct gui *gui) {
   int l=46+strlen(gui->editor->song->name)+intlen(gui->editor->block+1)+intlen(gui->editor->song->numblocks)+intlen(1)+intlen(gui->editor->song->blocks[gui->editor->block]->commandpages)+intlen(gui->editor->song->tempo)+intlen(gui->editor->song->ticksperline)+1;
   char *title=(char *)calloc(l, sizeof(char));
 
-  /* Refresh the main window widgets if the window exists */
-  if(gui->window_main!=NULL && GTK_IS_WINDOW(gui->window_main)) {
-    GtkLabel *status=GTK_LABEL(glade_xml_get_widget(gui->xml, 
-						    "gui_label_status"));
-
-    snprintf(section, 8+8, "Section %d/%d", gui->editor->section+1,
-	     gui->editor->song->numsections);
-    snprintf(playseq, 9+8, "Position %d/%d", gui->editor->position+1,
-	     gui->editor->song->playseqs[gui->editor->playseq]->length);
-    if(gui->editor->song->blocks[gui->editor->block]->name!=NULL)
-      snprintf(block, 6+10+32, "Block %d/%d: %s", gui->editor->block+1,
-	       gui->editor->song->numblocks,
-	       gui->editor->song->blocks[gui->editor->block]->name);
-    else
-      snprintf(block, 6+10+32, "Block %d/%d: ", gui->editor->block+1,
-	       gui->editor->song->numblocks);
-    gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(gui->xml,
-						      "gui_label_section")),
-		       section);
-    gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(gui->xml,
-						      "gui_label_position")),
-		       playseq);
-    gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(gui->xml,
-						      "gui_label_block")),
-		       block);
-    
-    /* Window title bar */
-    snprintf(title, l,
-	     "Tutka: %s - Block %d/%d - Cmd Page %d/%d - Tempo %d - TPL %d",
-	     gui->editor->song->name,
-	     gui->editor->block+1, gui->editor->song->numblocks,
-	     gui->editor->cmdpage+1,
-	     gui->editor->song->blocks[gui->editor->block]->commandpages,
-	     gui->editor->song->tempo, gui->editor->song->ticksperline);
-    gtk_window_set_title(GTK_WINDOW(gui->window_main), title); 
-
-    /* Status bar */
-    switch(gui->editor->player->mode) {
-    case MODE_PLAY_SONG:
-      gtk_label_set_text(status, "Playing song");
-      break;
-    case MODE_PLAY_BLOCK:
-      gtk_label_set_text(status, "Playing block");
-      break;
-    case MODE_IDLE:
-      gtk_label_set_text(status, "Stopped");
-      break;
-    }
-  }
+  /* Window title bar */
+  snprintf(title, l,
+	   "Tutka: %s - Block %d/%d - Cmd Page %d/%d - Tempo %d - TPL %d",
+	   gui->editor->song->name,
+	   gui->editor->block+1, gui->editor->song->numblocks,
+	   gui->editor->cmdpage+1,
+	   gui->editor->song->blocks[gui->editor->block]->commandpages,
+	   gui->editor->song->tempo, gui->editor->song->ticksperline);
+  gtk_window_set_title(GTK_WINDOW(gui->window_main), title); 
   free(title);
 }
 
+void gui_statusbar_refresh(struct gui *gui) {
+  GtkLabel *status=GTK_LABEL(glade_xml_get_widget(gui->xml, 
+						  "label_status"));
+  /* Status bar */
+  switch(gui->editor->player->mode) {
+  case MODE_PLAY_SONG:
+    gtk_label_set_text(status, "Playing song");
+    break;
+  case MODE_PLAY_BLOCK:
+    gtk_label_set_text(status, "Playing block");
+    break;
+  case MODE_IDLE:
+    gtk_label_set_text(status, "Stopped");
+    break;
+  }
+}
+
+void gui_info_refresh(struct gui *gui) {
+  char section[9+8], playseq[10+8], block[7+10+32];
+  
+  /* Refresh the main window widgets */
+  snprintf(section, 8+8, "Section %d/%d", gui->editor->section+1,
+	   gui->editor->song->numsections);
+  snprintf(playseq, 9+8, "Position %d/%d", gui->editor->position+1,
+	   gui->editor->song->playseqs[gui->editor->playseq]->length);
+  if(gui->editor->song->blocks[gui->editor->block]->name!=NULL)
+    snprintf(block, 6+10+32, "Block %d/%d: %s", gui->editor->block+1,
+	     gui->editor->song->numblocks,
+	     gui->editor->song->blocks[gui->editor->block]->name);
+  else
+    snprintf(block, 6+10+32, "Block %d/%d: ", gui->editor->block+1,
+	     gui->editor->song->numblocks);
+  gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(gui->xml,
+						    "label_section")),
+		     section);
+  gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(gui->xml,
+						    "label_position")),
+		     playseq);
+  gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(gui->xml,
+						    "label_block")), block);
+}
+
 void gui_timer_refresh(struct gui *gui, unsigned int seconds) {
   char time[6];
 
   /* Refresh the main window widgets if the window exists */
   if(gui->window_main!=NULL && GTK_IS_WINDOW(gui->window_main)) {
     snprintf(time, 6, "%.2d:%.2d", seconds/60, seconds%60);
-    gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(gui->xml, "gui_label_timer")), time);
+    gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(gui->xml, "label_timer")), time);
   }
 }
 
 /* Updates the current block information in the block list window */
 void gui_blocklist_info_refresh(struct gui *gui) {
-  GtkEntry *name=GTK_ENTRY(glade_xml_get_widget(gui->xml, "gui_entry_blocklist"));
+  GtkEntry *name=GTK_ENTRY(glade_xml_get_widget(gui->xml, "entry_blocklist"));
   char label[6+8];
   
   /* Show the number of the selected block */
   snprintf(label, 6+8, "Block %d/%d",
 	   gui->editor->block+1, gui->editor->song->numblocks);
   gtk_frame_set_label(GTK_FRAME(glade_xml_get_widget(gui->xml,
-						     "gui_frame_blocklist")),
+						     "frame_blocklist")),
 		      label);
   
   /* Show the name of the selected block */
@@ -421,18 +525,18 @@ void gui_blocklist_info_refresh(struct gui *gui) {
     gtk_entry_set_text(name, "");
   
   /* Show the properties of the selected block */
-  gtk_adjustment_set_value(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(glade_xml_get_widget(gui->xml, "gui_spinbutton_blocklist_tracks"))),
+  gtk_adjustment_set_value(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(glade_xml_get_widget(gui->xml, "spinbutton_blocklist_tracks"))),
 			   gui->editor->song->blocks[gui->editor->block]->tracks);
-  gtk_adjustment_set_value(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(glade_xml_get_widget(gui->xml, "gui_spinbutton_blocklist_length"))),
+  gtk_adjustment_set_value(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(glade_xml_get_widget(gui->xml, "spinbutton_blocklist_length"))),
 			   gui->editor->song->blocks[gui->editor->block]->length);
-  gtk_adjustment_set_value(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(glade_xml_get_widget(gui->xml, "gui_spinbutton_blocklist_commandpages"))),
+  gtk_adjustment_set_value(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(glade_xml_get_widget(gui->xml, "spinbutton_blocklist_commandpages"))),
 			   gui->editor->song->blocks[gui->editor->block]->commandpages);
 
   /* Make the delete button sensitive/insensitive */
   if(gui->editor->song->numblocks>1)
-    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "gui_button_blocklist_delete"), TRUE);
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_blocklist_delete"), TRUE);
   else
-    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "gui_button_blocklist_delete"), FALSE);
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_blocklist_delete"), FALSE);
   
 }
 
@@ -456,7 +560,7 @@ void gui_blocklist_refresh(struct gui *gui, gboolean recreate) {
   }
 
   /* Move back to the previously selected block */
-  treeview=GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "gui_treeview_blocklist"));
+  treeview=GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_blocklist"));
   treepath=gtk_tree_path_new_from_string(block);
   gtk_tree_selection_select_path(gtk_tree_view_get_selection(treeview),
 				 treepath);
@@ -474,41 +578,41 @@ void gui_playseq_info_refresh(struct gui *gui) {
   snprintf(position, 9+8, "Position %d/%d", gui->editor->position+1,
 	   gui->editor->song->playseqs[gui->editor->playseq]->length);
   snprintf(block, 6+8, "Block %d/%d", gui->editor->song->playseqs[gui->editor->playseq]->blocknumbers[gui->editor->position]+1, gui->editor->song->numblocks);
-  gtk_frame_set_label(GTK_FRAME(glade_xml_get_widget(gui->xml, "gui_frame_playseq")), position);
-  gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(gui->xml, "gui_label_playseq_block")), block);
+  gtk_frame_set_label(GTK_FRAME(glade_xml_get_widget(gui->xml, "frame_playseq")), position);
+  gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(gui->xml, "label_playseq_block")), block);
 
   /* Show playing sequence name */
   if(gui->editor->song->playseqs[gui->editor->playseq]->name!=NULL)
-    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(gui->xml, "gui_entry_playseq_name")), gui->editor->song->playseqs[gui->editor->playseq]->name);
+    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(gui->xml, "entry_playseq_name")), gui->editor->song->playseqs[gui->editor->playseq]->name);
   else
-    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(gui->xml, "gui_entry_playseq_name")), "");
+    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(gui->xml, "entry_playseq_name")), "");
     
   /* Set window title */
   snprintf(title, 24, "Playing Sequence %d/%d", gui->editor->playseq+1,
 	   gui->editor->song->numplayseqs);
-  gtk_window_set_title(GTK_WINDOW(glade_xml_get_widget(gui->xml, "gui_dialog_playseq")), title);
+  gtk_window_set_title(GTK_WINDOW(glade_xml_get_widget(gui->xml, "dialog_playseq")), title);
 
   /* Make the delete position button sensitive/insensitive */
   if(gui->editor->song->playseqs[gui->editor->playseq]->length>1)
-    gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(gui->xml, "gui_button_playseq_delete")), TRUE);
+    gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(gui->xml, "button_playseq_delete")), TRUE);
   else
-    gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(gui->xml, "gui_button_playseq_delete")), FALSE);
+    gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(gui->xml, "button_playseq_delete")), FALSE);
 
   /* Make the next/previous block buttons sensitive/insensitive */
   if(gui->editor->song->playseqs[gui->editor->playseq]->blocknumbers[gui->editor->position]>0)
-    gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(gui->xml, "gui_button_playseq_prev")), TRUE);
+    gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(gui->xml, "button_playseq_prev")), TRUE);
   else
-    gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(gui->xml, "gui_button_playseq_prev")), FALSE);
+    gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(gui->xml, "button_playseq_prev")), FALSE);
   if(gui->editor->song->playseqs[gui->editor->playseq]->blocknumbers[gui->editor->position]<gui->editor->song->numblocks-1)
-    gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(gui->xml, "gui_button_playseq_next")), TRUE);
+    gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(gui->xml, "button_playseq_next")), TRUE);
   else
-    gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(gui->xml, "gui_button_playseq_next")), FALSE);
+    gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(gui->xml, "button_playseq_next")), FALSE);
 
   /* Make the delete playing sequence button sensitive/insensitive */
   if(gui->editor->song->numplayseqs>1)
-    gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(gui->xml, "gui_button_playseq_deletethis")), TRUE);
+    gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(gui->xml, "button_playseq_deletethis")), TRUE);
   else
-    gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(gui->xml, "gui_button_playseq_deletethis")), FALSE);
+    gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(gui->xml, "button_playseq_deletethis")), FALSE);
 }
 
 void gui_playseq_refresh(struct gui *gui, gboolean recreate) {
@@ -531,33 +635,31 @@ void gui_playseq_refresh(struct gui *gui, gboolean recreate) {
     
   }
   /* Move back to the previously selected position */
-  treeview=GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "gui_treeview_playseq"));
+  treeview=GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_playseq"));
   treepath=gtk_tree_path_new_from_string(playseqpos);
   gtk_tree_selection_select_path(gtk_tree_view_get_selection(treeview),
 				 treepath);
   gtk_tree_view_scroll_to_cell(treeview, treepath, NULL, TRUE, 0.5, 0);
   gtk_tree_view_set_cursor(treeview, treepath, NULL, FALSE);
-  
-  gui_playseq_info_refresh(gui);
 }
 
 /* Refresh section list buttons */
 void gui_sectionlist_buttons_refresh(struct gui *gui) {
   /* Make the delete button sensitive/insensitive */
   if(gui->editor->song->numsections>1)
-    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "gui_button_sectionlist_delete"), TRUE);
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_sectionlist_delete"), TRUE);
   else
-    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "gui_button_sectionlist_delete"), FALSE);
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_sectionlist_delete"), FALSE);
 
   /* Make the next/previous buttons sensitive/insensitive */
   if(gui->editor->song->sections[gui->editor->section]>0)
-    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "gui_button_sectionlist_prev"), TRUE);
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_sectionlist_prev"), TRUE);
   else
-    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "gui_button_sectionlist_prev"), FALSE);
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_sectionlist_prev"), FALSE);
   if(gui->editor->song->sections[gui->editor->section]<gui->editor->song->numplayseqs-1)
-    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "gui_button_sectionlist_next"), TRUE);
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_sectionlist_next"), TRUE);
   else
-   gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "gui_button_sectionlist_next"), FALSE);
+   gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_sectionlist_next"), FALSE);
 }
 
 /* Refresh all section list widgets */
@@ -583,7 +685,7 @@ void gui_sectionlist_refresh(struct gui *gui, gboolean recreate) {
 
   /* Move back to the previously selected section */
   treeview=GTK_TREE_VIEW(glade_xml_get_widget(gui->xml,
-					      "gui_treeview_sectionlist"));
+					      "treeview_sectionlist"));
   treepath=gtk_tree_path_new_from_string(section);
   gtk_tree_selection_select_path(gtk_tree_view_get_selection(treeview),
 				 treepath);
@@ -593,7 +695,7 @@ void gui_sectionlist_refresh(struct gui *gui, gboolean recreate) {
   gui_sectionlist_buttons_refresh(gui);
 }
 
-void gui_sysexlist_refresh(struct gui *gui) {
+void gui_sysexlist_refresh(struct gui *gui, gboolean recreate) {
   /* Refresh the sysex list widgets */
   int i;
   char oldsysex[10], sysex[6+8];
@@ -603,35 +705,38 @@ void gui_sysexlist_refresh(struct gui *gui) {
 
   snprintf(oldsysex, 10, "%d", gui->sysexlist_sysex);
   
-  /* Recreate the SysEx list */
-  gtk_list_store_clear(gui->store_sysexlist);
-  for(i=0; i<gui->editor->song->numsysexes; i++) {
-    GtkTreeIter iter;
-    gtk_list_store_append(gui->store_sysexlist, &iter);
-    gtk_list_store_set(gui->store_sysexlist, &iter,
-		       0, i, 1, gui->editor->song->sysexes[i]->name, -1);
+  if(recreate) {
+    /* Recreate the SysEx list */
+    gtk_list_store_clear(gui->store_sysexlist);
+    for(i=0; i<gui->editor->song->numsysexes; i++) {
+      GtkTreeIter iter;
+      gtk_list_store_append(gui->store_sysexlist, &iter);
+      gtk_list_store_set(gui->store_sysexlist, &iter,
+			 0, i, 1, gui->editor->song->sysexes[i]->name, -1);
+    }
   }
-  
+
   /* Move back to the previously selected SysEx and make widgets sensitive */
   if(gui->sysexlist_sysex!=-1) {
-    gtk_tree_selection_select_path(gtk_tree_view_get_selection(GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "gui_treeview_sysexlist"))), gtk_tree_path_new_from_string(oldsysex));
+    gtk_tree_selection_select_path(gtk_tree_view_get_selection(GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_sysexlist"))), gtk_tree_path_new_from_string(oldsysex));
 
     /* Show the number of the selected SysEx message */
     snprintf(sysex, 6+8, "SysEx %d/%d", gui->sysexlist_sysex+1,
 	     gui->editor->song->numsysexes);
-    gtk_frame_set_label(GTK_FRAME(glade_xml_get_widget(gui->xml, "gui_frame_sysex_controls")), sysex);
-    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "gui_table_sysex1"), TRUE);
-    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "gui_button_sysex_send"), TRUE);
-    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "gui_button_sysex_receive"), TRUE);
-    /* gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "gui_checkbutton_sysex_autostop"), TRUE); */
-    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "gui_button_sysex_delete"), TRUE);
+    gtk_frame_set_label(GTK_FRAME(glade_xml_get_widget(gui->xml, "frame_sysex_controls")), sysex);
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(gui->xml, "spinbutton_sysex_length")), gui->editor->song->sysexes[gui->sysexlist_sysex]->length);
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "table_sysex1"), TRUE);
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_sysex_send"), TRUE);
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_sysex_receive"), TRUE);
+    /* gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "checkbutton_sysex_autostop"), TRUE); */
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_sysex_delete"), TRUE);
   } else {
-    gtk_frame_set_label(GTK_FRAME(glade_xml_get_widget(gui->xml, "gui_frame_sysex_controls")), "SysEx 0/0");
-    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "gui_table_sysex1"), FALSE);
-    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "gui_button_sysex_send"), FALSE);
-    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "gui_button_sysex_receive"), FALSE);
-    /* gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "gui_checkbutton_sysex_autostop"), FALSE); */
-    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "gui_button_sysex_delete"), FALSE);
+    gtk_frame_set_label(GTK_FRAME(glade_xml_get_widget(gui->xml, "frame_sysex_controls")), "SysEx 0/0");
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "table_sysex1"), FALSE);
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_sysex_send"), FALSE);
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_sysex_receive"), FALSE);
+    /* gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "checkbutton_sysex_autostop"), FALSE); */
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_sysex_delete"), FALSE);
   }
 }
 
@@ -639,38 +744,41 @@ void gui_trackvolumes_refresh(struct gui *gui) {
   /* Refresh the track volumes window widgets */
   int i;
   char track[4];
-  GtkWidget *w1, *w2, *w3, *w4;
-  GtkWidget *hbox=glade_xml_get_widget(gui->xml, "gui_hbox_trackvolumes");
+  GtkWidget *table=glade_xml_get_widget(gui->xml, "table_trackvolumes");
   
   /* Destroy all children */
-  gtk_container_foreach(GTK_CONTAINER(hbox), (GtkCallback)gtk_widget_destroy,
+  gtk_container_foreach(GTK_CONTAINER(table), (GtkCallback)gtk_widget_destroy,
 			NULL);
   
+  gtk_table_resize(GTK_TABLE(table), 2, gui->editor->song->maxtracks);
+
+  /* Create widgets for each channel */
   for(i=0; i<gui->editor->song->maxtracks; i++) {
-    w1=gtk_vbox_new(FALSE, 0);
-    gtk_box_pack_start(GTK_BOX(hbox), w1, TRUE, TRUE, 4);
-    
+    GtkWidget *w, *x;
+
     /* Volume sliders */
-    w2=(GtkWidget *)gtk_adjustment_new((gui->editor->song->trackvolumes[i])&127,
+    w=(GtkWidget *)gtk_adjustment_new((gui->editor->song->trackvolumes[i])&127,
 				       0, 127, 1, 1, 0);
-    g_signal_connect(G_OBJECT(w2), "value_changed",
-		     G_CALLBACK(on_gui_adjustment_trackvolumes_changed),
+    g_signal_connect(G_OBJECT(w), "value_changed",
+		     G_CALLBACK(on_adjustment_trackvolumes_changed),
 		     (gpointer)&gui->editor->song->trackvolumes[i]);
-    w3=gtk_vscale_new(GTK_ADJUSTMENT(w2));
-    gtk_range_set_inverted(GTK_RANGE(w3), TRUE);
-    gtk_scale_set_digits(GTK_SCALE(w3), 0);
-    gtk_box_pack_start(GTK_BOX(w1), w3, TRUE, TRUE, 4);
+    x=gtk_vscale_new(GTK_ADJUSTMENT(w));
+    gtk_range_set_inverted(GTK_RANGE(x), TRUE);
+    gtk_scale_set_digits(GTK_SCALE(x), 0);
+    gtk_table_attach(GTK_TABLE(table), x, i, i+1, 0, 1,
+		     GTK_EXPAND|GTK_FILL, GTK_EXPAND|GTK_FILL, 4, 4);
     
     /* Mute buttons */
     snprintf(track, 4, "%d", i+1);
-    w4=gtk_check_button_new_with_label(track);
+    w=gtk_check_button_new_with_label(track);
     if(gui->editor->song->trackvolumes[i]>127)
-      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w4), TRUE);
-    g_signal_connect(G_OBJECT(w4), "toggled",
-		       G_CALLBACK(on_gui_togglebutton_trackvolumes_toggled),
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
+    g_signal_connect(G_OBJECT(w), "toggled",
+		       G_CALLBACK(on_togglebutton_trackvolumes_toggled),
 		       (gpointer)&gui->editor->song->trackvolumes[i]);
-    gtk_box_pack_start(GTK_BOX(w1), w4, FALSE, FALSE, 4);
+    gtk_table_attach(GTK_TABLE(table), w, i, i+1, 1, 2,
+		     GTK_EXPAND|GTK_FILL, GTK_FILL, 4, 4);
   }
   
-  gtk_widget_show_all(hbox);
+  gtk_widget_show_all(table);
 }
