@@ -54,7 +54,7 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
   gboolean handled=FALSE;
 
   if(!(GTK_IS_ENTRY(GTK_WINDOW(widget)->focus_widget))) {
-    Tracker *tracker=(Tracker *)glade_xml_get_widget(gui->xml, "gui_tracker");
+    Tracker *tracker=(Tracker *)glade_xml_get_widget(gui->xml, "tracker");
     int shift=event->state & GDK_SHIFT_MASK;
     int ctrl=event->state & GDK_CONTROL_MASK;
     int alt=event->state & GDK_MOD1_MASK;
@@ -63,6 +63,7 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
       if(ctrl) {
 	switch (event->keyval) {
 	case 'b':
+	  /* CTRL-B: Selection mode on/off */
 	  if(tracker_is_in_selection_mode(tracker))
 	    tracker_mark_selection(tracker, FALSE);
 	  else
@@ -70,26 +71,31 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
 	  handled=TRUE;
 	  break;
 	case 'c':
+	  /* CTRL-C: Copy selection */
 	  if(tracker_is_in_selection_mode(tracker))
 	    tracker_mark_selection(tracker, FALSE);
-	  on_gui_menuitem_edit_copy_activate(NULL, gui);
+	  on_menuitem_edit_copy_activate(NULL, gui);
 	  handled=TRUE;
 	  break;
 	case 'x':
+	  /* CTRL-X: Cut selection */
 	  if(tracker_is_in_selection_mode(tracker))
 	    tracker_mark_selection(tracker, FALSE);
-	  on_gui_menuitem_edit_cut_activate(NULL, gui);
+	  on_menuitem_edit_cut_activate(NULL, gui);
 	  handled=TRUE;
 	  break;
 	case 'v':
-	  on_gui_menuitem_edit_paste_activate(NULL, gui);
+	  /* CTRL-V: Paste selection */
+	  on_menuitem_edit_paste_activate(NULL, gui);
 	  handled=TRUE;
 	  break;
 	case 'q':
+	  /* CTRL-Q: Quit */
 	  handled=TRUE;
 	  gtk_main_quit();
 	  break;
 	case 'k':
+	  /* CTRL-K: Clear until the end of the track */
 	  block_clear(song->blocks[editor->block],
 		      tracker->cursor_ch, tracker->patpos,
 		      tracker->cursor_ch,
@@ -98,22 +104,27 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
 	  handled=TRUE;
 	  break;
 	case 's':
+	  /* CTRL-S: Save song */
 	  song_save(song, editor->filename);
 	  handled=TRUE;
 	  break;
 	case GDK_F3:
-	  on_gui_menuitem_block_cut_activate(NULL, gui);
+	  /* CTRL-F3: Cut block */
+	  on_menuitem_block_cut_activate(NULL, gui);
 	  handled=TRUE;
 	  break;
 	case GDK_F4:
-	  on_gui_menuitem_block_copy_activate(NULL, gui);
+	  /* CTRL-F4: Copy block */
+	  on_menuitem_block_copy_activate(NULL, gui);
 	  handled=TRUE;
 	  break;
 	case GDK_F5:
-	  on_gui_menuitem_block_paste_activate(NULL, gui);
+	  /* CTRL-F5: Paste block */
+	  on_menuitem_block_paste_activate(NULL, gui);
 	  handled=TRUE;
 	  break;
 	case GDK_Left:
+	  /* CTRL-Left: Previous instrument */
 	  if(editor->instrument>0) {
 	    editor->instrument--;
 
@@ -124,6 +135,7 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
 	  handled=TRUE;
 	  break;
 	case GDK_Right:
+	  /* CTRL-Right: Previous instrument */
 	  editor->instrument++;
 
 	  /* Make sure the instrument exists */
@@ -133,7 +145,9 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
 	  handled=TRUE;
 	  break;
 	case GDK_ISO_Left_Tab:
-	  editor_set_commandpage_immediate(editor, editor->cmdpage+1);
+	  /* CTRL-Shift-Tab: Next command page */
+	  editor_set_commandpage(editor, editor->cmdpage+1);
+	  gui_window_title_refresh(gui);  
 	  handled=TRUE;
 	  break;
 	case '0':
@@ -146,42 +160,43 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
 	case '7':
 	case '8':
 	case '9':
+	  /* CTRL-0-9: Set space value */
 	  editor->space=event->keyval-'0';
-	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(gui->xml, "gui_spinbutton_space")), editor->space);
+	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(gui->xml, "spinbutton_space")), editor->space);
 	  handled=TRUE;
 	  break;
 	}
       } else if(shift) {
 	switch (event->keyval) {
 	case GDK_F3:
-	  on_gui_menuitem_track_cut_activate(NULL, gui);
+	  /* Shift-F3: Cut track */
+	  on_menuitem_track_cut_activate(NULL, gui);
 	  handled=TRUE;
 	  break;
 	case GDK_F4:
-	  on_gui_menuitem_track_copy_activate(NULL, gui);
+	  /* Shift-F4: Copy track */
+	  on_menuitem_track_copy_activate(NULL, gui);
 	  handled=TRUE;
 	  break;
 	case GDK_F5:
-	  on_gui_menuitem_track_paste_activate(NULL, gui);
+	  /* Shift-F5: Paste track */
+	  on_menuitem_track_paste_activate(NULL, gui);
 	  handled=TRUE;
 	  break;
 	case GDK_Left:
-	  if(editor->position>0) {
-	    editor->position--;
-	    gui_info_refresh(gui);
-	    gui_playseq_refresh(gui, FALSE);
-	  }
+	  /* Shift-Left: Next position */
+	  if(editor->position>0)
+	    editor_set_position(editor, editor->position-1);
 	  handled=TRUE;
 	  break;
 	case GDK_Right:
-	  if(editor->position<song->playseqs[editor->playseq]->length-1) {
-	    editor->position++;
-	    gui_info_refresh(gui);
-	    gui_playseq_refresh(gui, FALSE);
-	  }
+	  /* Shift-Right: Previous position */
+	  if(editor->position<song->playseqs[editor->playseq]->length-1)
+	    editor_set_position(editor, editor->position+1);
 	  handled=TRUE;
 	  break;
 	case GDK_ISO_Left_Tab:
+	  /* Shift-Tab: Previous track */
 	  if((tracker)->cursor_item>0) {
 	    (tracker)->cursor_item=0;
 	    tracker_step_cursor_channel(tracker, 0);
@@ -190,7 +205,9 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
 	  handled=TRUE;
 	  break;
 	case GDK_BackSpace:
+	  /* Shift-Backspace: Insert row */
 	  if(editor->edit) {
+	    int i;
 	    struct block *b=block_copy(song->blocks[editor->block],
 				       tracker->cursor_ch, tracker->patpos,
 				       tracker->cursor_ch,
@@ -199,9 +216,10 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
 			tracker->cursor_ch, tracker->patpos+1);
 	    block_set_note(song->blocks[editor->block],
 			   tracker->patpos, tracker->cursor_ch, 0, 0, 0);
-	    block_set_command_full(song->blocks[editor->block],
-				  tracker->patpos, tracker->cursor_ch,
-				  editor->cmdpage, 0, 0);
+	    for(i=0; i<song->blocks[editor->block]->commandpages; i++)
+	      block_set_command_full(song->blocks[editor->block],
+				     tracker->patpos, tracker->cursor_ch,
+				     i, 0, 0);
 	    block_free(b);
 	    tracker_redraw(tracker);
 	  }
@@ -213,13 +231,38 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
       } else if(alt) {
 	switch (event->keyval) {
 	case GDK_Left:
-	  editor_set_block_immediate(gui->editor, editor->block-1);
+	  /* Alt-Left: Previous block */
+	  editor_set_block(editor, editor->block-1);
 	  handled=TRUE;
 	  break;
 	case GDK_Right:
-	  editor_set_block_immediate(gui->editor, editor->block+1);
+	  /* Alt-Left: Next block */
+	  editor_set_block(editor, editor->block+1);
 	  handled=TRUE;
 	  break;
+	case '1':
+	case '2':
+	case '3':
+	case '4':
+	case '5':
+	case '6':
+	case '7':
+	case '8':
+	case '9':
+	case '0': {
+	  /* Alt+0-9: Switch channels on/off */
+	  int channel=9;
+	  if(event->keyval>='1' && event->keyval<='9')
+	    channel=event->keyval-'1';
+	  if(channel<editor->song->maxtracks) {
+	    if(editor->song->trackvolumes[channel]>127)
+	      editor->song->trackvolumes[channel]&=127;
+	    else
+	      editor->song->trackvolumes[channel]|=128;
+	    gui_trackvolumes_refresh(gui);
+	  }
+	  break;
+	}
 	default:
 	  break;
 	}
@@ -230,6 +273,7 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
 	  char data=-1;
 	  switch(tracker->cursor_item) {
 	  case 0:
+	    /* Editing notes */
 	    switch(event->keyval) {
 	    case 'z':
 	      data=1;
@@ -328,6 +372,7 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
 	      break;
 	    case GDK_BackSpace:
 	      {
+		/* Backspace: delete line */
 		struct block *b=block_copy(song->blocks[editor->block],
 					   tracker->cursor_ch,
 					   tracker->patpos+1,
@@ -349,23 +394,26 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
 	      break;
 	    }
 	    if(data!=-1) {
-	      block_set_note(song->blocks[editor->block], tracker->patpos,
+	      /* Set note and refresh */
+	      block_set_note(song->blocks[editor->block], editor->line,
 			     tracker->cursor_ch, editor->octave, data,
 			     editor->instrument+1);
-	      tracker_redraw_row(tracker, tracker->patpos);
+	      tracker_redraw_row(tracker, editor->line);
+
 	      if(editor->chord) {
+		/* Chord mode: Go to next channel */
 		tracker_step_cursor_channel(tracker, 1);
 		if(event->keyval!=chord_previous_keyval) {
 		  editor->chordstatus++;
 		  chord_previous_keyval=event->keyval;
 		}
 	      } else
-		editor_set_line_immediate(gui->editor,
-					  editor->line+editor->space);
+		editor_set_line(editor, editor->line+editor->space);
 	      handled=TRUE;
 	    }
 	    break;
 	  case 1:
+	    /* Editing instrument */
 	    if(event->keyval>='0' && event->keyval<='9')
 	      data=event->keyval-'0';
 	    else if(event->keyval>='a' && event->keyval<='z')
@@ -374,12 +422,12 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
 	      data=0;
 
 	    if(data!=-1) {
+	      /* Set instrument and refresh */
 	      block_set_instrument(song->blocks[editor->block],
-				   tracker->patpos, tracker->cursor_ch,
+				   editor->line, tracker->cursor_ch,
 				   data);
-	      tracker_redraw_row(tracker, tracker->patpos);
-	      editor_set_line_immediate(gui->editor,
-					editor->line+editor->space);
+	      tracker_redraw_row(tracker, editor->line);
+	      editor_set_line(editor, editor->line+editor->space);
 	      handled=TRUE;
 	    }
 	    break;
@@ -387,6 +435,7 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
 	  case 3:
 	  case 4:
 	  case 5:
+	    /* Editing effects */
 	    if(event->keyval>='0' && event->keyval<='9')
 	      data=event->keyval-'0';
 	    else if(event->keyval>='a' && event->keyval<='f')
@@ -395,17 +444,18 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
 	      data=0;
 
 	    if(data!=-1) {
+	      /* Set effect and refresh */
 	      block_set_command(song->blocks[editor->block],
-				tracker->patpos, tracker->cursor_ch,
+				editor->line, tracker->cursor_ch,
 				editor->cmdpage, tracker->cursor_item-2, data);
-	      tracker_redraw_row(tracker, tracker->patpos);
-	      editor_set_line_immediate(gui->editor,
-					editor->line+editor->space);
+	      tracker_redraw_row(tracker, editor->line);
+	      editor_set_line(editor, editor->line+editor->space);
 	      handled=TRUE;
 	    }
 	    break;
 	  }
 	}
+
 	switch (event->keyval) {
 	case GDK_space:
 	  /* If the song is playing, stop */
@@ -415,7 +465,7 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
 	  } else {
 	    /* Otherwise toggle edit mode */
 	    editor->edit^=1;
-	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(gui->xml, "gui_checkbutton_edit")), editor->edit);
+	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(gui->xml, "checkbutton_edit")), editor->edit);
 	  }
 	  handled=TRUE;
 	  break;
@@ -430,7 +480,8 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
 	case GDK_F9:
 	case GDK_F10:
 	case GDK_F11:
-	  gtk_option_menu_set_history(GTK_OPTION_MENU(glade_xml_get_widget(gui->xml, "gui_optionmenu_keyboard")), event->keyval-GDK_F1);
+	  /* Set active keyboard octave */
+	  gtk_option_menu_set_history(GTK_OPTION_MENU(glade_xml_get_widget(gui->xml, "optionmenu_keyboard")), event->keyval-GDK_F1);
 	  editor->octave=event->keyval-GDK_F1;
 	  handled=TRUE;
 	  break;
@@ -592,41 +643,49 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
 	  note=29;
 	  break;
 	case GDK_Down:
-	  editor_set_line_immediate(gui->editor, editor->line+1);
+	  /* Down: Go down */
+	  editor_set_line(editor, editor->line+1);
 	  handled=TRUE;
 	  break;
 	case GDK_Up:
-	  editor_set_line_immediate(gui->editor, editor->line-1);
+	  /* Up: Go up */
+	  editor_set_line(editor, editor->line-1);
 	  handled=TRUE;
 	  break;
 	case GDK_Left:
+	  /* Left: Go left */
 	  tracker_step_cursor_item(tracker, -1);
 	  handled=TRUE;
 	  break;
 	case GDK_Right:
+	  /* Right: Go right */
 	  tracker_step_cursor_item(tracker, 1);
 	  handled=TRUE;
 	  break;
 	case GDK_Tab:
+	  /* Tab: Next track */
 	  (tracker)->cursor_item=0;
 	  tracker_step_cursor_channel(tracker, 1);
 	  handled=TRUE;
 	  break;
 	case GDK_Home:
-	  editor_set_line_immediate(gui->editor, 0);
+	  /* End: Go to the beginning of block */
+	  editor_set_line(editor, 0);
 	  handled=TRUE;
 	  break;
 	case GDK_End:
-	  editor_set_line_immediate(gui->editor,
-				    song->blocks[editor->block]->length-1);
+	  /* End: Go to the end of block */
+	  editor_set_line(editor, song->blocks[editor->block]->length-1);
 	  handled=TRUE;
 	  break;
 	case GDK_Page_Down:
-	  editor_set_line_immediate(gui->editor, editor->line+8);
+	  /* Page down: Go down 8 lines */
+	  editor_set_line(editor, editor->line+8);
 	  handled=TRUE;
 	  break;
 	case GDK_Page_Up:
-	  editor_set_line_immediate(gui->editor, editor->line-8);
+	  /* Page up: Go up 8 lines */
+	  editor_set_line(editor, editor->line-8);
 	  handled=TRUE;
 	  break;
 	}
@@ -694,7 +753,7 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
 	    tracker_step_cursor_channel(tracker, -1);
 	    editor->chordstatus--;
 	    if(editor->chordstatus==0)
-	      editor_set_line_immediate(editor, editor->line+editor->space);
+	      editor_set_line(editor, editor->line+editor->space);
 	    handled=TRUE;
 	    break;
 	  }
