@@ -334,27 +334,27 @@ struct MMD1Block *MMD1Block_parse(unsigned char *base, int offset) {
 struct MMD0exp *MMD0exp_parse(unsigned char *base, int offset) {
   unsigned char *data=base+offset, *data2, *data3;
   struct MMD0exp *mmd0exp=(struct MMD0exp *)calloc(1, sizeof(struct MMD0exp));
-  int i;
+  int i, entrysize;
 
   if(getlong(data)!=0)
     mmd0exp->nextmod=(struct MMD0 *)MMD2_parse(base, getlong(data));
   mmd0exp->s_ext_entries=getword(data+8);
-  mmd0exp->s_ext_entrsz=sizeof(struct InstrExt); /* getword(data+10); */
+  mmd0exp->s_ext_entrsz=sizeof(struct InstrExt);
+  entrysize=getword(data+10);
   mmd0exp->exp_smp=(struct InstrExt *)calloc(mmd0exp->s_ext_entries,
 					     sizeof(struct InstrExt));
-  /* mmd0exp->s_ext_entrsz); */
   data2=base+getlong(data+4);
   for(i=0; i<mmd0exp->s_ext_entries; i++) {
-    mmd0exp->exp_smp[i].hold=getbyte(data2+i*mmd0exp->s_ext_entrsz);
-    mmd0exp->exp_smp[i].decay=getbyte(data2+i*mmd0exp->s_ext_entrsz+1);
-    mmd0exp->exp_smp[i].suppress_midi_off=getbyte(data2+i*mmd0exp->s_ext_entrsz+2);
-    mmd0exp->exp_smp[i].finetune=getbyte(data2+i*mmd0exp->s_ext_entrsz+3);
-    if(mmd0exp->s_ext_entrsz>4) {
-      mmd0exp->exp_smp[i].default_pitch=getbyte(data2+i*mmd0exp->s_ext_entrsz+4);
-      mmd0exp->exp_smp[i].instr_flags=getbyte(data2+i*mmd0exp->s_ext_entrsz+5);
-      mmd0exp->exp_smp[i].long_midi_preset=getbyte(data2+i*mmd0exp->s_ext_entrsz+6);
-      if(mmd0exp->s_ext_entrsz>8)
-	mmd0exp->exp_smp[i].output_device=getbyte(data2+i*mmd0exp->s_ext_entrsz+8);
+    mmd0exp->exp_smp[i].hold=getbyte(data2+i*entrysize);
+    mmd0exp->exp_smp[i].decay=getbyte(data2+i*entrysize+1);
+    mmd0exp->exp_smp[i].suppress_midi_off=getbyte(data2+i*entrysize+2);
+    mmd0exp->exp_smp[i].finetune=getbyte(data2+i*entrysize+3);
+    if(entrysize>4) {
+      mmd0exp->exp_smp[i].default_pitch=getbyte(data2+i*entrysize+4);
+      mmd0exp->exp_smp[i].instr_flags=getbyte(data2+i*entrysize+5);
+      mmd0exp->exp_smp[i].long_midi_preset=getbyte(data2+i*entrysize+6);
+      if(entrysize>8)
+	mmd0exp->exp_smp[i].output_device=getbyte(data2+i*entrysize+8);
     }
   }
   mmd0exp->annolen=getlong(data+16);
@@ -439,96 +439,3 @@ struct MMD0exp *MMD0exp_parse(unsigned char *base, int offset) {
 
   return mmd0exp;
 }
-
-void MMD2_print(struct MMD2 *mmd, char *path) {
-  char *notestrings[] = {
-    "---",
-    "C-1", "C#1", "D-1", "D#1", "E-1", "F-1", "F#1", "G-1", "G#1", "A-1", "A#1", "B-1",
-    "C-2", "C#2", "D-2", "D#2", "E-2", "F-2", "F#2", "G-2", "G#2", "A-2", "A#2", "B-2",
-    "C-3", "C#3", "D-3", "D#3", "E-3", "F-3", "F#3", "G-3", "G#3", "A-3", "A#3", "B-3",
-    "C-4", "C#4", "D-4", "D#4", "E-4", "F-4", "F#4", "G-4", "G#4", "A-4", "A#4", "B-4",
-    "C-5", "C#5", "D-5", "D#5", "E-5", "F-5", "F#5", "G-5", "G#5", "A-5", "A#5", "B-5",
-    "C-6", "C#6", "D-6", "D#6", "E-6", "F-6", "F#6", "G-6", "G#6", "A-6", "A#6", "B-6",
-    "C-7", "C#7", "D-7", "D#7", "E-7", "F-7", "F#7", "G-7", "G#7", "A-7", "A#7", "B-7",
-    "C-8", "C#8", "D-8", "D#8", "E-8", "F-8", "F#8", "G-8", "G#8", "A-8", "A#8", "B-8",
-    "C-9", "C#9", "D-9", "D#9", "E-9", "F-9", "F#9", "G-9", "G#9", "A-9", "A#9", "B-9",
-    "C-A", "C#A", "D-A", "D#A", "E-A", "F-A", "F#A", "G-A", "G#A", "A-A", "A#A", "B-A"
-  };
-
-  FILE *file;
-  struct MMD2song *song;
-  int i, j, k;
-
-  song=mmd->song;
-
-  if(path)
-    file=fopen(path, "r+");
-  else
-    file=stdout;
-
-  if(file) {
-    fprintf(file, "======= Song: %s\n\n", mmd->expdata->songname);
-    fprintf(file, "#  Instrument name                    Repeat RepLen Mc Mpst Vol Trns Hold Dec\n");
-    
-    fprintf(file, "\nDefault Tempo = %d/%d, Play Transpose = %d\n\n", song->deftempo, song->tempo2, song->playtransp);
-    fprintf(file, "Master Volume = %d, Track Volumes:", song->mastervol);
-    for(i=0; i<64; i++) {
-      if(!(i%8))
-	fprintf(file, "\n");
-      fprintf(file, "%2d: %3d  ", i, song->trackvols[i]);
-    }
-    
-    fprintf(file, "\n\n======================================================================\n\n");
-
-    for(i=0; i<song->numpseqs; i++) {
-      struct PlaySeq *playseq;
-
-      playseq=song->playseqtable[i];
-
-      fprintf(file, "Playing Sequence List #%d (length = %d): %s", i+1, playseq->length, playseq->name);
-      for(j=0; j<playseq->length; j++) {
-	if(!(j%8))
-	  fprintf(file, "\n");
-	fprintf(file, "%3d ", playseq->seq[j]);
-      }
-      fprintf(file, "\n\n");
-    }
-
-    fprintf(file, "Section List (length = %d):\n", song->songlen);
-    for(i=0; i<song->songlen; i++) {
-      fprintf(file, "%.3d: %.3d %s\n", i+1, song->sectiontable[i]+1, song->playseqtable[song->sectiontable[i]]->name);
-    }
-
-    fprintf(file, "\n======================================================================\n\n");
-
-    for(i=0; i<song->numblocks; i++) {
-      struct MMD1Block *block;
-      char *blockdata;
-      char datastring[6];
-      datastring[5]=0;
-
-      block=mmd->blockarr[i];
-      blockdata=(char *)block+sizeof(struct MMD1Block);
-
-      fprintf(file, "Block #%3d: %3d tracks, %3d lines\n", i, block->numtracks, block->lines);
-      fprintf(file, "=================================\n");
-
-      for(j=0; j<block->lines; j++) {
-	fprintf(file, "%.3d ", j);
-	for(k=0; k<block->numtracks; k++) {
-	  datastring[0]='0'+blockdata[4*block->numtracks*j+4*k+1]%0x3f;
-	  datastring[1]='0'+((blockdata[4*block->numtracks*j+4*k+2]%0xf0)>>4);
-	  datastring[2]='0'+blockdata[4*block->numtracks*j+4*k+2]%0x0f;
-	  datastring[3]='0'+((blockdata[4*block->numtracks*j+4*k+3]%0xf0)>>4);
-	  datastring[4]='0'+blockdata[4*block->numtracks*j+4*k+3]%0x0f;
-	  fprintf(file, "%s %s ",
-		  notestrings[blockdata[4*block->numtracks*j+4*k]%0x7f],
-		  datastring);
-	}
-	fprintf(file, "\n");
-      }
-	
-      fprintf(file, "\n");
-    }
-  }
-}
