@@ -135,7 +135,7 @@ struct playseq *playseq_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) {
       }
       cur=cur->next;
     }
-  } else
+  } else if(cur->type!=XML_COMMENT_NODE)
     fprintf(stderr, "XML error: expected playseq, got %s\n", cur->name);
 
   return playseq;
