@@ -512,9 +512,8 @@ struct song *song_convert_MMD2(struct MMD2 *mmd) {
     song->numsections=mmd->song->songlen;
     song->sections=(unsigned int *)calloc(song->numsections,
 					    sizeof(unsigned int));
-    for(i=0; i<song->numsections; i++) {
-      /* FIX: Where to get these? */
-    }
+    for(i=0; i<song->numsections; i++)
+      song->sections[i]=mmd->song->sectiontable[i];
     /* Playing sequences */
     song->numplayseqs=mmd->song->numpseqs;
     song->playseqs=(struct playseq **)calloc(song->numplayseqs,
@@ -545,8 +544,12 @@ struct song *song_convert_MMD2(struct MMD2 *mmd) {
 			  MMD1Block->info->pagetable->num_pages+1);
       else
 	block=block_alloc(MMD1Block->numtracks, MMD1Block->lines+1, 1);
-
+      
       song->blocks[i]=block;
+
+      if(MMD1Block->info->blockname!=NULL)
+	block->name=strdup(MMD1Block->info->blockname);
+
       for(j=0; j<block->length; j++)
 	for(k=0; k<block->tracks; k++) {
 	  block->notes[(j*block->tracks+k)*2]=notes[(j*block->tracks+k)*4];
@@ -635,7 +638,7 @@ struct song *song_convert_MMD2(struct MMD2 *mmd) {
   return song;
 }
 
-/* Transposes a song */
+/* Transposes all blocks in a song */
 void song_transpose(struct song *song, int instruments, int halfnotes) {
   int i;
 
@@ -649,6 +652,35 @@ void song_transpose(struct song *song, int instruments, int halfnotes) {
 		    song->blocks[i]->tracks-1, song->blocks[i]->length-1);
 }
 
+/* Expands/shrinks all blocks in a song */
+void song_expandshrink(struct song *song, int factor) {
+  int i;
+
+  if(song==NULL) {
+    fprintf(stderr, "song_expandshrink() called with null song\n");
+    return;
+  }
+
+  for(i=0; i<song->numblocks; i++)
+    block_expandshrink(song->blocks[i], factor, 0, 0,
+		    song->blocks[i]->tracks-1, song->blocks[i]->length-1);
+}
+
+/* Changes an instrument to another in all blocks of a song */
+void song_changeinstrument(struct song *song, int from, int to, int swap) {
+  int i;
+
+  if(song==NULL) {
+    fprintf(stderr, "song_changeinstrument() called with null song\n");
+    return;
+  }
+
+  for(i=0; i<song->numblocks; i++)
+    block_changeinstrument(song->blocks[i], from, to, swap, 0, 0,
+			   song->blocks[i]->tracks-1,
+			   song->blocks[i]->length-1);
+}
+
 /* Converts MMD command values at given location */
 void song_convert_commands(unsigned char *command,
 			   struct MMDMIDICmd3x *mmdcmd3x) {
@@ -698,7 +730,7 @@ void song_convert_commands(unsigned char *command,
 	break;
       case 0xfe:
 	/* Stops the song playing */
-	command[0]=COMMAND_STOP;
+	command[0]=COMMAND_TEMPO;
 	command[1]=0;
 	break;
       case 0xff:
@@ -729,7 +761,7 @@ void song_convert_commands(unsigned char *command,
   case 0x3e:
   case 0x3f:
     if(mmdcmd3x->ctrlr_types[command[0]-0x31]==MCS_TYPE_STD_MSB)
-      command[0]=mmdcmd3x->ctrlr_numbers[command[0]-0x31];
+      command[0]=mmdcmd3x->ctrlr_numbers[command[0]-0x31]+0x80;
     break;
   default:
     break;
@@ -754,11 +786,7 @@ struct song *song_load(char *filename) {
       mmd=MMD2_load(filename);
       song=song_convert_MMD2(mmd);
       MMD2_free(mmd);
-      /* If it didn't succeed either just allocate a new song */
-      if(song!=NULL)
-	return song;
-      else
-	return song_alloc();
+      return song;
     } else {
       /* Try loading it as XML */     
       xmlDocPtr doc;
@@ -773,7 +801,7 @@ struct song *song_load(char *filename) {
       return song_parse(doc, NULL, cur);
     }
   } else
-    return song_alloc();
+    return NULL;
 }
 
 /* Parses a song element in an XML file */
@@ -860,7 +888,7 @@ struct song *song_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) {
 	      if(temp2!=NULL)
 		song->sections[number]=atoi(temp2->content);
 	      song->numsections=number+1;
-	    } else
+	    } else if(temp->type!=XML_COMMENT_NODE)
 	      fprintf(stderr, "XML error: expected section, got %s\n",
 		      temp->name);
 	  }
@@ -958,7 +986,7 @@ struct song *song_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) {
 	      if(temp2!=NULL)
 		song->trackvolumes[track]=atoi(temp2->content);
 	      song->maxtracks=track+1;
-	    } else 
+	    } else if(temp->type!=XML_COMMENT_NODE)
 	      fprintf(stderr, "XML error: expected section, got %s\n",
 		      temp->name);
 	  }
@@ -1001,7 +1029,7 @@ struct song *song_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) {
       }
       cur=cur->next;
     }
-  } else
+  } else if(cur->type!=XML_COMMENT_NODE)
     fprintf(stderr, "XML error: expected song, got %s\n", cur->name);
 
   return song;
@@ -1091,3 +1119,101 @@ void song_save(struct song *song, char *filename) {
   xmlSaveFile(filename, doc);
   xmlFreeDoc(doc);
 }
+
+
+void song_print(struct song *song, char *path) {
+  char *notestrings[] = {
+    "---",
+    "C-1", "C#1", "D-1", "D#1", "E-1", "F-1", "F#1", "G-1", "G#1", "A-1", "A#1", "B-1",
+    "C-2", "C#2", "D-2", "D#2", "E-2", "F-2", "F#2", "G-2", "G#2", "A-2", "A#2", "B-2",
+    "C-3", "C#3", "D-3", "D#3", "E-3", "F-3", "F#3", "G-3", "G#3", "A-3", "A#3", "B-3",
+    "C-4", "C#4", "D-4", "D#4", "E-4", "F-4", "F#4", "G-4", "G#4", "A-4", "A#4", "B-4",
+    "C-5", "C#5", "D-5", "D#5", "E-5", "F-5", "F#5", "G-5", "G#5", "A-5", "A#5", "B-5",
+    "C-6", "C#6", "D-6", "D#6", "E-6", "F-6", "F#6", "G-6", "G#6", "A-6", "A#6", "B-6",
+    "C-7", "C#7", "D-7", "D#7", "E-7", "F-7", "F#7", "G-7", "G#7", "A-7", "A#7", "B-7",
+    "C-8", "C#8", "D-8", "D#8", "E-8", "F-8", "F#8", "G-8", "G#8", "A-8", "A#8", "B-8",
+    "C-9", "C#9", "D-9", "D#9", "E-9", "F-9", "F#9", "G-9", "G#9", "A-9", "A#9", "B-9",
+    "C-A", "C#A", "D-A", "D#A", "E-A", "F-A", "F#A", "G-A", "G#A", "A-A", "A#A", "B-A"
+  };
+  char *instrumentstrings="0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
+  FILE *file;
+  int i, j, k;
+
+  if(path)
+    file=fopen(path, "r+");
+  else
+    file=stdout;
+
+  if(file) {
+    fprintf(file, "======= Song: %s\n\n", song->name);
+    fprintf(file, "#  Instrument name                               Mc Mpst Vol Trns Hold\n");
+    for(i=0; i<song->numinstruments; i++) {
+      fprintf(file, "%.2d %-45s %2d %4d %3d %4d %4d\n",
+	      i+1, song->instruments[i]->name,
+	      song->instruments[i]->midichannel,
+	      song->instruments[i]->midipreset,
+	      song->instruments[i]->defaultvelocity,
+	      song->instruments[i]->transpose,
+	      song->instruments[i]->hold);
+    }
+
+    fprintf(file, "\nDefault Tempo = %d/%d\n\n", song->tempo, song->ticksperline);
+    fprintf(file, "Master Volume = %d, Track Volumes:", song->mastervolume);
+    for(i=0; i<song->maxtracks; i++) {
+      if(!(i%8))
+	fprintf(file, "\n");
+      fprintf(file, "%2d: %3d  ", i+1, song->trackvolumes[i]);
+    }
+    
+    fprintf(file, "\n\n======================================================================\n\n");
+
+    for(i=0; i<song->numplayseqs; i++) {
+      struct playseq *playseq;
+
+      playseq=song->playseqs[i];
+
+      fprintf(file, "Playing Sequence List #%d (length = %d): %s", i+1, playseq->length, playseq->name);
+      for(j=0; j<playseq->length; j++) {
+	if(!(j%8))
+	  fprintf(file, "\n");
+	fprintf(file, "%3d ", playseq->blocknumbers[j]+1);
+      }
+      fprintf(file, "\n\n");
+    }
+
+    fprintf(file, "Section List (length = %d):\n", song->numsections);
+    for(i=0; i<song->numsections; i++) {
+      fprintf(file, "%.3d: %.3d %s\n", i+1, song->sections[i]+1, song->playseqs[song->sections[i]]->name);
+    }
+
+    fprintf(file, "\n======================================================================\n\n");
+
+    for(i=0; i<song->numblocks; i++) {
+      struct block *block;
+      char datastring[6];
+      datastring[5]=0;
+
+      block=song->blocks[i];
+
+      fprintf(file, "Block #%3d: %3d tracks, %3d lines\n", i+1, block->tracks, block->length);
+      fprintf(file, "=================================\n");
+
+      for(j=0; j<block->length; j++) {
+	fprintf(file, "%.3d ", j);
+	for(k=0; k<block->tracks; k++) {
+	  datastring[0]=instrumentstrings[block->notes[(block->tracks*j+k)*2+1]&0x3f];
+	  datastring[1]=instrumentstrings[(block->commands[(block->tracks*j+k)*2]&0xf0)>>4];
+	  datastring[2]=instrumentstrings[block->commands[(block->tracks*j+k)*2]&0x0f];
+	  datastring[3]=instrumentstrings[(block->commands[(block->tracks*j+k)*2+1]&0xf0)>>4];
+	  datastring[4]=instrumentstrings[block->commands[(block->tracks*j+k)*2+1]&0x0f];
+	  fprintf(file, "%s %s ",
+		  notestrings[block->notes[(block->tracks*j+k)*2]&0x7f],
+		  datastring);
+	}
+	fprintf(file, "\n");
+      }
+	
+      fprintf(file, "\n");
+    }
+  }
+}
