@@ -143,6 +143,8 @@ void tracker_set_pattern (Tracker *t, struct block *pattern) {
     if(pattern != NULL) {
       if(t->patpos >= pattern->length)
 	t->patpos = pattern->length-1;
+      if(t->cursor_ch >= pattern->tracks)
+	t->cursor_ch = pattern->tracks-1;
       g_signal_emit(GTK_OBJECT(t), tracker_signals[PATPOS_SIGNAL], t->patpos, pattern->length, t->disp_rows);
     }
     gtk_widget_queue_draw(GTK_WIDGET(t));
